
function dados = Envia_sms(obj_serial,tlfn,IP)
%% Envia um SMS ao telefone cadastrado
%
% O SMS conter, no cabealho UDH, a porta de ativao da aplicao pelo
% PUSH REGISTRY na unidade mvel e no corpo o IP de acesso ao servidor.
%
% Inicialmente, deve-se enviar ao mdulo os comandos de indicao do
% formato de envio de mensagens SMS. Atravs destes comandos, indicamos ao
% mdulo que usaremos o formato PDU - Protocol Data Unit, padro de envio a
% baixo nvel de mensagens SMS.
%
% Aps, damos o comando de escrita do SMS. O comando de envio da mensagem
% consiste em enviar primeiramente um 
%
%         AT+CMGW=<numero_de_octetos_do_SMS>
%
% e enviar a mensagem em seguida. O encerramento da entrada dos dados 
% dado com o caracter CTRL+z
%
% O comando anterior salva a mensagem no storage padro do mdulo, aps a
% sua execuo, deve-se dar o comando de envio da mensagem, passando como
% parmetro a posio de memria em que a mensagem foi salva.
%

%% Inicializao do SMS

global PDU_header;
PDU_header = true;

global tempo_pausa;

% Comandos de inicializao
cmd = {preencheZero('at+csms=1'),...
		preencheZero('at+cnmi=3,3,3,2,1'),...
		preencheZero('at^smgo=1'),...
		preencheZero('at+cmgf=0'),...
		preencheZero('at+cmee=2'),...
		};

% Enviando os comandos
imp('Configurando o mdulo para o envio do SMS',2)
for R = 1:size(cmd,2)

	imp(['Entrando na funo Envia_cmd(obj_serial,cmd1{' num2str(R) '});'],0)
	Envia_cmd(obj_serial,cmd{R});
	[dados,cod] = Recebe_retorno(obj_serial);
    % Se recebeu a tecla de parada
	if cod == 60
        imp('Parando a execuo do programa, saindo da funo Envia_SMS',1)
        dados = 'q';
        return;
    % Seno, se chegou qualquer coisa diferente de OK
    elseif cod ~= 0
		imp(['Mensagem de cdigo ' num2str(cod) ' recebida. Aguardando alguns instantes e reiniciando a aplicao.'],1)
		pause(tempo_pausa);
		dados = '';
		return;
	end

end

%% Fabricando o SMS
% Criamos a funo make_PDU para fabricar o SMS seguindo o padro GSM 03.40
imp('Fabricando o SMS',2)
[SMS,SMSlength] = make_PDU(tlfn,IP);
imp(['Dados que sero enviados: ' SMS],0)
imp(['Nmero de bytes: ' num2str(SMSlength)],0)

%% Salvando a mensagem na memria do mdulo
imp('Salvando a mensagem na memria',2)
comando = ['at+cmgw=' num2str(SMSlength)];

imp(['Enviando o comando:' comando],0)
Envia_cmd(obj_serial,comando);

% Como no temos controle das mensagens que o mdulo ir retornar, qualquer
% mensagem pode chegar a qualquer momento. Uma mensagem em particular  a
% mensagem de encerramento de conexo. Porm, como o mdulo provavelmente
% est aguardando os dados para o SMS, setamos a varivel a seguir. Assim,
% indicamos  funo recebe_retorno que caso chegue uma mensagem de
% encerramento de conexo, no  para fazer nada com ela, e em seguida
% deve-se voltar a aguardar a mensagem
global modulo_aguardando_dados;
modulo_aguardando_dados = true;

[dados,cod] = Recebe_retorno(obj_serial);
% Se recebeu a tecla de parada
if cod == 60
	imp('Parando a execuo do programa, saindo da funo Envia_SMS',1)
	dados = 'q';
	return;
% Seno, se chegou qualquer coisa diferente
elseif cod ~= 20
	imp(['Mensagem de cdigo ' num2str(cod) ' recebida. Aguardando alguns instantes e reiniciando a aplicao.'],1)
	pause(tempo_pausa);
	dados = '';
	return;
end

modulo_aguardando_dados = false;

imp(['Enviando o texto da mensagem: ' SMS],0)
fprintf(obj_serial,[SMS 26]);
[posicao,cod] = Recebe_retorno(obj_serial);
% Se recebeu a tecla de parada
if cod == 60
	imp('Parando a execuo do programa, saindo da funo Envia_SMS',1)
	dados = 'q';
	return;
% Seno, se chegou qualquer coisa diferente
elseif cod ~= 25
	imp(['Mensagem de cdigo ' num2str(cod) ' recebida. Aguardando alguns instantes e reiniciando a aplicao.'],1)
	pause(tempo_pausa);
	dados = '';
	return;
end
imp('Aguardando a mensagem de OK',0)
[dados,cod] = Recebe_retorno(obj_serial);
% Se recebeu a tecla de parada
if cod == 60
	imp('Parando a execuo do programa, saindo da funo Envia_SMS',1)
	dados = 'q';
	return;
% Seno, se chegou qualquer coisa diferente
elseif cod ~= 0
	imp(['Mensagem de cdigo ' num2str(cod) ' recebida. Aguardando alguns instantes e reiniciando a aplicao.'],1)
	pause(tempo_pausa);
	dados = '';
	return;
end

%% Enviando a mensagem para o telefone

imp('Enviando a mensagem salva para o telefone',2)
comando = ['at+cmss=' posicao];

imp(['Enviando o comando de envio do SMS:' comando],0)
Envia_cmd(obj_serial,comando);
[dados,cod] = Recebe_retorno(obj_serial);
% Se recebeu a tecla de parada
if cod == 60
	imp('Parando a execuo do programa, saindo da funo Envia_SMS',1)
	dados = 'q';
	return;
% Seno, se chegou qualquer coisa diferente
elseif cod ~= 0
	imp(['Mensagem de cdigo ' num2str(cod) ' recebida. Aguardando alguns instantes e reiniciando a aplicao.'],1)
	pause(tempo_pausa);
	dados = '';
	return;
end
