function [UDL, UD] = make_UD(dado,portaD,portaO)

%% UDH - User Data Header
% IEI - Information Element Identifier
%       Indica o tipo do elemento do cabealho (ver pg. 63 do GSM 03.40)
IEI = '05';
% IEIDL - Length of Information Element Data
%         Indica o comprimento em octetos do campo IEID
IEIDL = '04';
% IEID - Information Element Data
%        O dado do elemento em questo
IEID1 = dec2hex(portaD,4);
IEID2 = dec2hex(portaO,4);

% Junta tudo num UDH
UDH = [IEI IEIDL IEID1 IEID2];

%% UDHL - User Data Header Length
% Comprimento (nmero de octetos) do campo UDH
UDHL = dec2hex(size(UDH,2)/2,2);

%% Junta os dois e forma o Header
header = [UDHL UDH];
imp(['Cabealho UDH: ' header],0);

%% UD - User Data
% Enviaremos o IP de acesso do servidor atravs do campo UD.
% Este campo  codificado em 7 bits usando o alfabeto GSM (ver GSM 03.40).
% A seguir, realizamos o procedimento de codificao.

%% Converte o IP para hexadecimal
% Transmitiremos as informaes em hexadecimal
T = 1:size(dado,2);
IP = dec2hex(dado(T),2);

%% Concatena o IP num vetor
IP_frase = [];
for R = 1:size(IP,1)
	IP_frase = [IP_frase IP(R,:)];
end

%% Reverte o IP dois a dois
% O endereo IP ser revertido de octeto a octeto (dois caracteres hexa).
% Desta forma, o primeiro byte ser o ltimo do vetor
IP_frase_inv = [];
for R = size(IP_frase,2):-2:2
	IP_frase_inv = [IP_frase_inv IP_frase(R-1:R)];
end

%% Converte para uma frase binria
% Converte cada byte para uma frase binria e concatena tudo num vetor
% binrio. Cada byte em hexa  convertido para uma palavra binria de sete
% bits.
IP_frase_inv_bin = [];
for R = 1:2:size(IP_frase_inv,2)
	IP_frase_inv_bin = [IP_frase_inv_bin de2bi(hex2dec(num2str(IP_frase_inv(R:R+1))),7,'left-msb')];
end

%% Pega de 8 em 8 bits, de trs para frente
% Aps esta linha, teremos uma matriz de 8 colunas por X linhas. Cada linha
% ter uma palavra binria.

% Como pegamos e convertemos de 7 em 7 bits, pode ser necessrio, preencher
% com bits para que a string feche com um nmero divisvel por 8 
preenche = 8 - mod(size(IP_frase_inv_bin,2),8);
if preenche ~= 0
	IP_frase_inv_bin = [zeros(1,preenche) IP_frase_inv_bin];
end

% Pega de oito em oito bits de trs pra frente
r = 1;
for R = size(IP_frase_inv_bin,2):-8:1
	
	IP_bin_matriz(r,:) = IP_frase_inv_bin(R - 7:R);
	r = r+1;
	
end

%% Pega a palavra hexa resultante
% Converte tudo para hexadecimal
IP_final = [];
for R = 1:size(IP_bin_matriz,1)
	IP_final = [IP_final dec2hex(bi2de(IP_bin_matriz(R,:),'left-msb'),2)];
end
imp(['IP codificado em 7 bits: ' IP_final],0);

%% Monta o quadro, concatenando o header com o corpo da mensagem
UD = [header IP_final];

%% Pega o tamanho de todo o campo
% O tamanho de todo o campo UD  medido em septetos.
% No cabealho, que  codificado em octetos, dividimos por 2 porque est
% codificado em hexadecimal, depois multiplicamos por 8 para termos o
% nmero de bits e depois multiplicamos por 7 para termos o nmero de
% septetos.
% No corpo da mensagem, pegamos a string resultante da converso da string
% hexa para bit, retiramos os bytes inseridos, e dividimos por  sete.
% Assim, temos tambm o nmero de septetos.
UDL = dec2hex((size(header,2)/2)*8/7 + (size(IP_frase_inv_bin,2) - preenche)/7,2);


